@@ -19,13 +19,16 @@ from nbxmpp.const import InviteType
 from nbxmpp.const import StatusCode
 from nbxmpp.errors import StanzaError
 from nbxmpp.modules.dataforms import SimpleDataForm
+from nbxmpp.modules.muc.util import MucInfoResult
 from nbxmpp.namespaces import Namespace
 from nbxmpp.protocol import JID
 from nbxmpp.protocol import Message
 from nbxmpp.protocol import Presence
 from nbxmpp.structs import CommonError
+from nbxmpp.structs import CommonResult
 from nbxmpp.structs import DiscoInfo
 from nbxmpp.structs import MessageProperties
+from nbxmpp.structs import MucConfigResult
 from nbxmpp.structs import PresenceProperties
 from nbxmpp.structs import StanzaHandler
 from nbxmpp.structs import VoiceRequest
@@ -40,8 +43,6 @@ from gajim.common.const import MUCJoinedState
 from gajim.common.events import MucAdded
 from gajim.common.events import MucDecline
 from gajim.common.events import MucInvitation
-from gajim.common.helpers import get_default_muc_config
-from gajim.common.helpers import get_group_chat_nick
 from gajim.common.helpers import to_user_string
 from gajim.common.modules.base import BaseModule
 from gajim.common.modules.bits_of_binary import store_bob_data
@@ -50,6 +51,8 @@ from gajim.common.modules.contacts import GroupchatParticipant
 from gajim.common.structs import MUCData
 from gajim.common.structs import MUCPresenceData
 from gajim.common.util.datetime import utc_now
+from gajim.common.util.muc import get_default_muc_config
+from gajim.common.util.muc import get_group_chat_nick
 
 log = logging.getLogger('gajim.c.m.muc')
 
@@ -160,7 +163,7 @@ class MUC(BaseModule):
         return self._muc_service_jid is not None
 
     @property
-    def service_jid(self):
+    def service_jid(self) -> JID | None:
         return self._muc_service_jid
 
     def pass_disco(self, info: DiscoInfo) -> None:
@@ -405,7 +408,7 @@ class MUC(BaseModule):
 
     def configure_room(self, room_jid: str) -> None:
         self._nbxmpp('MUC').request_config(room_jid,
-                                           callback=self._on_room_config)
+                                           callback=self._on_room_config)  # type: ignore
 
     def _on_room_config(self, task: Task) -> None:
         try:
@@ -418,6 +421,7 @@ class MUC(BaseModule):
             room.notfiy('room-config-failed', error_text)
             return
 
+        assert isinstance(result, MucConfigResult)
         self._log.info('Configure room: %s', result.jid)
 
         muc_data = self._mucs[result.jid]
@@ -452,6 +456,7 @@ class MUC(BaseModule):
             room.notfiy('room-config-failed', error_text)
             return
 
+        assert isinstance(result, CommonResult)
         self._con.get_module('Discovery').disco_muc(
             result.jid, callback=self._on_disco_result_after_config)
 
@@ -462,6 +467,7 @@ class MUC(BaseModule):
             self._log.info('Disco %s failed: %s', error.jid, error.get_text())
             return
 
+        assert isinstance(result, MucInfoResult)
         jid = result.info.jid
         muc_data = self._mucs[jid]
         self._room_join_complete(muc_data)
@@ -496,7 +502,7 @@ class MUC(BaseModule):
             status=message)
 
     def _on_error_presence(self,
-                           _con: types.xmppClient,
+                           _con: types.NBXMPPClient,
                            stanza: Presence,
                            properties: PresenceProperties
                            ) -> None:
@@ -506,11 +512,6 @@ class MUC(BaseModule):
         if muc_data is None:
             return
 
-        if properties.jid.resource != muc_data.nick:
-            self._log.warning('Unknown error presence')
-            self._log.warning(stanza)
-            return
-
         room = self._get_contact(room_jid)
 
         if muc_data.state == MUCJoinedState.JOINING:
@@ -563,7 +564,7 @@ class MUC(BaseModule):
             room.notify('room-presence-error', event)
 
     def _on_muc_user_presence(self,
-                              _con: types.xmppClient,
+                              _con: types.NBXMPPClient,
                               stanza: Presence,
                               properties: PresenceProperties
                               ) -> None:
@@ -575,12 +576,14 @@ class MUC(BaseModule):
             return
 
         muc_data = self._mucs[room_jid]
+        assert properties.jid is not None
         occupant = self._get_contact(properties.jid, groupchat=True)
         room = self._get_contact(properties.jid.bare)
+        assert isinstance(room, GroupchatContact)
 
         timestamp = utc_now()
 
-        if properties.is_muc_destroyed:
+        if properties.muc_destroyed is not None:
             self._log.info('MUC destroyed: %s', room_jid)
             self._set_muc_state(room_jid, MUCJoinedState.NOT_JOINED)
             self._con.get_module('Bookmarks').remove(room_jid)
@@ -597,6 +600,7 @@ class MUC(BaseModule):
             return
 
         if properties.is_nickname_changed:
+            assert properties.muc_user is not None
             if properties.is_muc_self_presence:
                 muc_data.nick = properties.muc_user.nick
                 self._con.get_module('Bookmarks').modify(muc_data.jid,
@@ -778,12 +782,28 @@ class MUC(BaseModule):
             app.storage.events.store(occupant.room, event)
             signals_and_events.append(('user-status-show-changed', event))
 
+        if occupant.hats != presence.hats:
+            if presence.hats:
+                hats = [h.title for h in presence.hats.get_hats()]
+            else:
+                hats = None
+            event = events.MUCUserHatsChanged(
+                timestamp=timestamp,
+                is_self=properties.is_muc_self_presence,
+                nick=occupant.name,
+                hats=hats,
+            )
+
+            app.storage.events.store(occupant, event)
+            app.storage.events.store(occupant.room, event)
+            signals_and_events.append(('user-hats-changed', event))
+
         occupant.update_presence(presence)
         for signal, event in signals_and_events:
             occupant.notify(signal, event)
 
     def _on_muc_normal_user_message(self,
-                                    _con: types.xmppClient,
+                                    _con: types.NBXMPPClient,
                                     stanza: Presence,
                                     properties: MessageProperties) -> None:
         if not properties.muc_user or not properties.muc_user.jid:
@@ -858,7 +878,7 @@ class MUC(BaseModule):
             del self._rejoin_timeouts[room_jid]
 
     def _on_subject_change(self,
-                           _con: types.xmppClient,
+                           _con: types.NBXMPPClient,
                            _stanza: Message,
                            properties: MessageProperties
                            ) -> None:
@@ -925,13 +945,14 @@ class MUC(BaseModule):
                 muc_data.jid)
 
     def _on_voice_request(self,
-                          _con: types.xmppClient,
+                          _con: types.NBXMPPClient,
                           _stanza: Message,
                           properties: MessageProperties
                           ) -> None:
         if not properties.is_voice_request:
             return
 
+        assert properties.jid is not None
         room = self._get_contact(properties.jid.bare)
         assert isinstance(room, GroupchatContact)
         assert properties.voice_request is not None
@@ -963,7 +984,7 @@ class MUC(BaseModule):
         self._voice_requests[contact].remove(voice_request)
 
     def _on_captcha_challenge(self,
-                              _con: types.xmppClient,
+                              _con: types.NBXMPPClient,
                               _stanza: Message,
                               properties: MessageProperties
                               ) -> None:
@@ -975,6 +996,7 @@ class MUC(BaseModule):
             self._log.warning('Ignore captcha challenge received from MAM')
             raise nbxmpp.NodeProcessed
 
+        assert properties.jid is not None
         muc_data = self._mucs.get(properties.jid)
         if muc_data is None:
             return
@@ -1018,7 +1040,7 @@ class MUC(BaseModule):
         self._set_muc_state(room_jid, MUCJoinedState.JOINING)
         self._nbxmpp('MUC').send_captcha(room_jid,
                                          form_node,
-                                         callback=self._on_captcha_result)
+                                         callback=self._on_captcha_result)  # type: ignore
 
     def _on_captcha_result(self, task: Task) -> None:
         try:
@@ -1033,7 +1055,7 @@ class MUC(BaseModule):
             room.notify('room-captcha-error', error_text)
 
     def _on_config_change(self,
-                          _con: types.xmppClient,
+                          _con: types.NBXMPPClient,
                           _stanza: Message,
                           properties: MessageProperties
                           ) -> None:
@@ -1059,7 +1081,7 @@ class MUC(BaseModule):
         raise nbxmpp.NodeProcessed
 
     def _on_invite_or_decline(self,
-                              _con: types.xmppClient,
+                              _con: types.NBXMPPClient,
                               _stanza: Message,
                               properties: MessageProperties
                               ) -> None:
@@ -1079,6 +1101,7 @@ class MUC(BaseModule):
         if properties.muc_invite is not None:
             data = properties.muc_invite
             contact = self._get_contact(data.muc, groupchat=True)
+            assert isinstance(contact, GroupchatContact)
 
             self._log.info('Invite from: %s, to: %s', data.from_, data.muc)
 
@@ -1090,8 +1113,8 @@ class MUC(BaseModule):
             self._con.get_module('Discovery').disco_muc(
                 data.muc,
                 request_vcard=True,
-                callback=self._on_disco_result_after_invite,
-                user_data=data)
+                callback=self._on_disco_result_after_invite,  # type: ignore
+                user_data=data)  # type: ignore
 
             raise nbxmpp.NodeProcessed
 
