@@ -1,16 +1,6 @@
 # This file is part of Gajim.
 #
-# Gajim is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published
-# by the Free Software Foundation; version 3 only.
-#
-# Gajim is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Gajim. If not, see <http://www.gnu.org/licenses/>.
+# SPDX-License-Identifier: GPL-3.0-only
 
 from __future__ import annotations
 
@@ -18,7 +8,6 @@ from typing import Any
 
 import logging
 
-from gi.repository import Gdk
 from gi.repository import Gtk
 from gi.repository import GtkSource
 from nbxmpp.errors import StanzaError
@@ -28,6 +17,7 @@ from nbxmpp.task import Task
 
 from gajim.common import app
 from gajim.common import ged
+from gajim.common.ged import EventHelper
 from gajim.common.helpers import to_user_string
 from gajim.common.i18n import _
 
@@ -35,38 +25,36 @@ from gajim.gtk.builder import get_builder
 from gajim.gtk.dataform import DataFormWidget
 from gajim.gtk.dialogs import ConfirmationDialog
 from gajim.gtk.dialogs import DialogButton
-from gajim.gtk.dialogs import ErrorDialog
-from gajim.gtk.dialogs import WarningDialog
-from gajim.gtk.util import EventHelper
-from gajim.gtk.util import get_source_view_style_scheme
+from gajim.gtk.dialogs import SimpleDialog
+from gajim.gtk.util.styling import get_source_view_style_scheme
+from gajim.gtk.widgets import GajimAppWindow
 
-log = logging.getLogger('gajim.gtk.pep_config')
+log = logging.getLogger("gajim.gtk.pep_config")
 
 
-class PEPConfig(Gtk.ApplicationWindow, EventHelper):
+class PEPConfig(GajimAppWindow, EventHelper):
     def __init__(self, account: str) -> None:
-        Gtk.ApplicationWindow.__init__(self)
-        EventHelper.__init__(self)
-        self.set_application(app.app)
-        self.set_position(Gtk.WindowPosition.CENTER)
-        self.set_show_menubar(False)
-        self.set_name('PEPConfig')
-        self.set_default_size(700, 800)
-        self.set_resizable(True)
-        self.set_transient_for(app.window)
+        GajimAppWindow.__init__(
+            self,
+            name="PEPConfig",
+            title=_("PEP Service Configuration (%s)") % account,
+            default_width=700,
+            default_height=800,
+        )
 
-        self._ui = get_builder('pep_config.ui')
-        self.add(self._ui.stack)
+        EventHelper.__init__(self)
+
+        self._ui = get_builder("pep_config.ui")
+        self.set_child(self._ui.stack)
 
         self.account = account
-        self.set_title(_('PEP Service Configuration (%s)') % self.account)
         self._client = app.get_client(account)
 
         self._result_node: Node | None = None
         self._dataform_widget: DataFormWidget | None = None
 
         source_manager = GtkSource.LanguageManager.get_default()
-        lang = source_manager.get_language('xml')
+        lang = source_manager.get_language("xml")
         self._ui.items_view.get_buffer().set_language(lang)
 
         style_scheme = get_source_view_style_scheme()
@@ -76,21 +64,23 @@ class PEPConfig(Gtk.ApplicationWindow, EventHelper):
         self._init_services()
 
         selection = self._ui.services_treeview.get_selection()
-        self._changed_handler = selection.connect(
-            'changed', self._on_services_selection_changed)
+        self._connect(selection, "changed", self._on_services_selection_changed)
 
-        self.register_events([
-            ('style-changed', ged.GUI1, self._on_style_changed)
-        ])
+        self._connect(
+            self._ui.show_content_button, "clicked", self._on_show_content_clicked
+        )
+        self._connect(self._ui.delete_button, "clicked", self._on_delete_button_clicked)
+        self._connect(
+            self._ui.configure_button, "clicked", self._on_configure_button_clicked
+        )
+        self._connect(self._ui.items_back_button, "clicked", self._on_back_clicked)
+        self._connect(self._ui.config_back_button, "clicked", self._on_back_clicked)
+        self._connect(self._ui.save_button, "clicked", self._on_save_config_clicked)
 
-        self.show_all()
-        self.connect('key-press-event', self._on_key_press)
-        self.connect('destroy', self._on_destroy)
-        self._ui.connect_signals(self)
+        self.register_events([("style-changed", ged.GUI1, self._on_style_changed)])
 
-    def _on_destroy(self, *args: Any) -> None:
-        selection = self._ui.services_treeview.get_selection()
-        selection.disconnect(self._changed_handler)
+    def _cleanup(self) -> None:
+        self.unregister_events()
         app.check_finalize(self)
 
     def _on_style_changed(self, *args: Any) -> None:
@@ -98,13 +88,7 @@ class PEPConfig(Gtk.ApplicationWindow, EventHelper):
         if style_scheme is not None:
             self._ui.items_view.get_buffer().set_style_scheme(style_scheme)
 
-    def _on_key_press(self, _widget: Gtk.Widget, event: Gdk.EventKey) -> None:
-        if event.keyval == Gdk.KEY_Escape:
-            self.destroy()
-
-    def _on_services_selection_changed(self,
-                                       _selection: Gtk.TreeSelection
-                                       ) -> None:
+    def _on_services_selection_changed(self, _selection: Gtk.TreeSelection) -> None:
 
         self._ui.configure_button.set_sensitive(True)
         self._ui.show_content_button.set_sensitive(True)
@@ -116,23 +100,24 @@ class PEPConfig(Gtk.ApplicationWindow, EventHelper):
         self.treestore.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         self._ui.services_treeview.set_model(self.treestore)
 
-        col = Gtk.TreeViewColumn(title=_('Service'))
+        col = Gtk.TreeViewColumn(title=_("Service"))
         col.set_sort_column_id(0)
         self._ui.services_treeview.append_column(col)
 
         cellrenderer_text = Gtk.CellRendererText()
         col.pack_start(cellrenderer_text, True)
-        col.add_attribute(cellrenderer_text, 'text', 0)
+        col.add_attribute(cellrenderer_text, "text", 0)
 
         jid = self._client.get_own_jid().bare
-        self._client.get_module('Discovery').disco_items(
-            jid, callback=self._items_received)
+        self._client.get_module("Discovery").disco_items(
+            jid, callback=self._items_received
+        )
 
     def _items_received(self, task: Task) -> None:
         try:
             result = task.finish()
         except StanzaError as error:
-            ErrorDialog('Error', to_user_string(error))
+            SimpleDialog(_("Error"), to_user_string(error))
             return
 
         jid = result.jid.bare
@@ -151,19 +136,19 @@ class PEPConfig(Gtk.ApplicationWindow, EventHelper):
         node = model[iter_][0]
 
         def _delete():
-            self._client.get_module('PubSub').delete(
-                node,
-                callback=self._on_node_delete,
-                user_data=node)
+            self._client.get_module("PubSub").delete(
+                node, callback=self._on_node_delete, user_data=node
+            )
 
         ConfirmationDialog(
-            _('Delete'),
-            _('Delete PEP node?'),
-            _('Do you really want to delete this PEP node?'),
-            [DialogButton.make('Cancel'),
-             DialogButton.make('Delete',
-                               callback=_delete)],
-            transient_for=self).show()
+            _("Delete Node?"),
+            _("Do you really want to delete this PEP node?"),
+            [
+                DialogButton.make("Cancel"),
+                DialogButton.make("Delete", callback=_delete),
+            ],
+            transient_for=self.window,
+        )
 
     def _on_node_delete(self, task: Task) -> None:
         node = task.get_user_data()
@@ -171,11 +156,11 @@ class PEPConfig(Gtk.ApplicationWindow, EventHelper):
         try:
             task.finish()
         except StanzaError as error:
-            WarningDialog(
-                _('PEP node was not removed'),
-                _('PEP node %(node)s was not removed:\n%(message)s') % {
-                    'node': node,
-                    'message': error})
+            SimpleDialog(
+                _("PEP Node Not Removed"),
+                _("PEP node %(node)s was not removed:\n%(message)s")
+                % {"node": node, "message": error},
+            )
             return
 
         model = self._ui.services_treeview.get_model()
@@ -196,42 +181,42 @@ class PEPConfig(Gtk.ApplicationWindow, EventHelper):
         assert iter_
         node = model[iter_][0]
 
-        self._client.get_module('PubSub').get_node_configuration(
-            node,
-            callback=self._on_pep_config_received)
+        self._client.get_module("PubSub").get_node_configuration(
+            node, callback=self._on_pep_config_received
+        )
 
     def _on_pep_config_received(self, task: Task) -> None:
         try:
             result = task.finish()
         except Exception:
-            log.exception('Failed to retrieve config')
+            log.exception("Failed to retrieve config")
             return
 
         if self._dataform_widget is not None:
             self._ui.form_box.remove(self._dataform_widget)
-            self._dataform_widget.destroy()
 
         self._result_node = result.node
 
-        form = dataforms.extend_form(node=result.form)  # pyright: ignore
+        form = dataforms.extend_form(node=result.form)
         self._dataform_widget = DataFormWidget(form)
         self._dataform_widget.set_propagate_natural_height(True)
-        self._dataform_widget.show_all()
-        self._ui.form_box.add(self._dataform_widget)
+        self._dataform_widget.set_visible(True)
+        self._ui.form_box.append(self._dataform_widget)
         self._ui.form_label.set_text(result.node)
 
-        self._ui.stack.set_visible_child_name('config')
+        self._ui.stack.set_visible_child_name("config")
 
     def _on_save_config_clicked(self, _button: Gtk.Button) -> None:
         assert self._dataform_widget is not None
         form = self._dataform_widget.get_submit_form()
-        self._client.get_module('PubSub').set_node_configuration(
-            self._result_node, form)
+        self._client.get_module("PubSub").set_node_configuration(
+            self._result_node, form
+        )
 
-        self._ui.stack.set_visible_child_name('overview')
+        self._ui.stack.set_visible_child_name("overview")
 
     def _on_back_clicked(self, _button: Gtk.Button) -> None:
-        self._ui.stack.set_visible_child_name('overview')
+        self._ui.stack.set_visible_child_name("overview")
 
     def _on_show_content_clicked(self, _button: Gtk.Button) -> None:
         selection = self._ui.services_treeview.get_selection()
@@ -242,16 +227,15 @@ class PEPConfig(Gtk.ApplicationWindow, EventHelper):
         assert iter_
         node = model[iter_][0]
 
-        self._client.get_module('PubSub').request_items(
-            node,
-            callback=self._on_pep_items_received,
-            user_data=node)
+        self._client.get_module("PubSub").request_items(
+            node, callback=self._on_pep_items_received, user_data=node
+        )
 
     def _on_pep_items_received(self, task: Task) -> None:
         try:
             result = task.finish()
         except Exception:
-            log.exception('Failed to retrieve items')
+            log.exception("Failed to retrieve items")
             return
 
         parent_node = task.get_user_data()
@@ -268,4 +252,4 @@ class PEPConfig(Gtk.ApplicationWindow, EventHelper):
             buf = self._ui.items_view.get_buffer()
             buf.insert_at_cursor(str(node))
 
-        self._ui.stack.set_visible_child_name('items')
+        self._ui.stack.set_visible_child_name("items")
